@@ -19,37 +19,54 @@
 extern struct StatList user_list;
 extern struct AATree user_tree;
 extern struct StatList pool_list;
+extern struct StatList peer_pool_list;
 extern struct StatList database_list;
+extern struct StatList peer_list;
 extern struct StatList autodatabase_idle_list;
 extern struct StatList login_client_list;
 extern struct Slab *client_cache;
 extern struct Slab *server_cache;
 extern struct Slab *db_cache;
+extern struct Slab *peer_cache;
+extern struct Slab *peer_pool_cache;
 extern struct Slab *pool_cache;
 extern struct Slab *user_cache;
 extern struct Slab *iobuf_cache;
+extern struct Slab *outstanding_request_cache;
+extern struct Slab *var_list_cache;
+extern struct Slab *server_prepared_statement_cache;
+extern PgPreparedStatement *prepared_statements;
 
+PgDatabase *find_peer(int peer_id);
 PgDatabase *find_database(const char *name);
+PgDatabase *find_or_register_database(PgSocket *connection, const char *name);
 PgUser *find_user(const char *name);
 PgPool *get_pool(PgDatabase *, PgUser *);
+PgPool *get_peer_pool(PgDatabase *);
 PgSocket *compare_connections_by_time(PgSocket *lhs, PgSocket *rhs);
-bool evict_connection(PgDatabase *db)		_MUSTCHECK;
-bool evict_user_connection(PgUser *user)	_MUSTCHECK;
-bool find_server(PgSocket *client)		_MUSTCHECK;
+bool evict_connection(PgDatabase *db)           _MUSTCHECK;
+bool evict_user_connection(PgUser *user)        _MUSTCHECK;
+bool find_server(PgSocket *client)              _MUSTCHECK;
 bool life_over(PgSocket *server);
-bool release_server(PgSocket *server)		/* _MUSTCHECK */;
-bool finish_client_login(PgSocket *client)	_MUSTCHECK;
-bool check_fast_fail(PgSocket *client)		_MUSTCHECK;
+bool release_server(PgSocket *server) /* _MUSTCHECK */;
+bool finish_client_login(PgSocket *client)      _MUSTCHECK;
+bool check_fast_fail(PgSocket *client)          _MUSTCHECK;
 
 PgSocket *accept_client(int sock, bool is_unix) _MUSTCHECK;
 void disconnect_server(PgSocket *server, bool notify, const char *reason, ...) _PRINTF(3, 4);
 void disconnect_client(PgSocket *client, bool notify, const char *reason, ...) _PRINTF(3, 4);
+void disconnect_client_sqlstate(PgSocket *client, bool notify, const char *sqlstate, const char *reason);
 
+PgDatabase * add_peer(const char *name, int peer_id) _MUSTCHECK;
 PgDatabase * add_database(const char *name) _MUSTCHECK;
 PgDatabase *register_auto_database(const char *name);
 PgUser * add_user(const char *name, const char *passwd) _MUSTCHECK;
 PgUser * add_db_user(PgDatabase *db, const char *name, const char *passwd) _MUSTCHECK;
 PgUser * force_user(PgDatabase *db, const char *username, const char *passwd) _MUSTCHECK;
+bool add_outstanding_request(PgSocket *client, char type, ResponseAction action) _MUSTCHECK;
+bool pop_outstanding_request(PgSocket *client, char *types, bool *skip);
+bool clear_outstanding_requests_until_sync(PgSocket *server) _MUSTCHECK;
+bool queue_fake_response(PgSocket *client, char request_type) _MUSTCHECK;
 
 PgUser * add_pam_user(const char *name, const char *passwd) _MUSTCHECK;
 
@@ -62,14 +79,12 @@ bool use_client_socket(int fd, PgAddr *addr, const char *dbname, const char *use
 		       const char *client_end, const char *std_string, const char *datestyle, const char *timezone,
 		       const char *password,
 		       const char *scram_client_key, int scram_client_key_len,
-		       const char *scram_server_key, int scram_server_key_len)
-			_MUSTCHECK;
+		       const char *scram_server_key, int scram_server_key_len) _MUSTCHECK;
 bool use_server_socket(int fd, PgAddr *addr, const char *dbname, const char *username, uint64_t ckey, int oldfd, int linkfd,
 		       const char *client_end, const char *std_string, const char *datestyle, const char *timezone,
 		       const char *password,
 		       const char *scram_client_key, int scram_client_key_len,
-		       const char *scram_server_key, int scram_server_key_len)
-			_MUSTCHECK;
+		       const char *scram_server_key, int scram_server_key_len) _MUSTCHECK;
 
 void activate_client(PgSocket *client);
 
