@@ -45,6 +45,13 @@ struct AATree user_tree;
  */
 struct AATree pam_user_tree;
 
+/*
+ * The global prepared statement cache, which deduplicates prepared statements
+ * sent by the clients statements by storing every unique prepared statement
+ * only once.
+ */
+PgPreparedStatement *prepared_statements = NULL;
+
 /*
  * client and server objects will be pre-allocated
  * they are always in either active or free lists
@@ -60,7 +67,9 @@ struct Slab *peer_pool_cache;
 struct Slab *pool_cache;
 struct Slab *user_cache;
 struct Slab *iobuf_cache;
+struct Slab *outstanding_request_cache;
 struct Slab *var_list_cache;
+struct Slab *server_prepared_statement_cache;
 
 /*
  * libevent may still report events when event_del()
@@ -94,6 +103,7 @@ static void construct_client(void *obj)
 	sbuf_init(&client->sbuf, client_proto);
 	client->vars.var_list = slab_alloc(var_list_cache);
 	client->state = CL_FREE;
+	client->client_prepared_statements = NULL;
 }
 
 static void construct_server(void *obj)
@@ -105,6 +115,8 @@ static void construct_server(void *obj)
 	sbuf_init(&server->sbuf, server_proto);
 	server->vars.var_list = slab_alloc(var_list_cache);
 	server->state = SV_FREE;
+	server->server_prepared_statements = NULL;
+	statlist_init(&server->outstanding_requests, "outstanding_requests");
 }
 
 /* compare string with PgUser->name, for usage with btree */
@@ -132,6 +144,7 @@ void init_objects(void)
 	peer_cache = slab_create("peer_cache", sizeof(PgDatabase), 0, NULL, USUAL_ALLOC);
 	peer_pool_cache = slab_create("peer_pool_cache", sizeof(PgPool), 0, NULL, USUAL_ALLOC);
 	pool_cache = slab_create("pool_cache", sizeof(PgPool), 0, NULL, USUAL_ALLOC);
+	outstanding_request_cache = slab_create("outstanding_request_cache", sizeof(OutstandingRequest), 0, NULL, USUAL_ALLOC);
 
 	if (!user_cache || !db_cache || !peer_cache || !peer_pool_cache || !pool_cache)
 		fatal("cannot create initial caches");
@@ -149,9 +162,40 @@ void init_caches(void)
 	server_cache = slab_create("server_cache", sizeof(PgSocket), 0, construct_server, USUAL_ALLOC);
 	client_cache = slab_create("client_cache", sizeof(PgSocket), 0, construct_client, USUAL_ALLOC);
 	iobuf_cache = slab_create("iobuf_cache", IOBUF_SIZE, 0, do_iobuf_reset, USUAL_ALLOC);
-	var_list_cache = slab_create("var_list_cache", sizeof(struct PStr*) * get_num_var_cached(), 0, NULL, USUAL_ALLOC);
+	var_list_cache = slab_create("var_list_cache", sizeof(struct PStr *) * get_num_var_cached(), 0, NULL, USUAL_ALLOC);
+	server_prepared_statement_cache = slab_create("server_prepared_statement_cache", sizeof(PgServerPreparedStatement), 0, NULL, USUAL_ALLOC);
 }
 
+/* free all memory related to the given client */
+static void client_free(PgSocket *client)
+{
+	free_client_prepared_statements(client);
+	varcache_clean(&client->vars);
+	slab_free(var_list_cache, client->vars.var_list);
+	slab_free(client_cache, client);
+}
+
+/* free all memory related to the given server */
+static void server_free(PgSocket *server)
+{
+	struct List *el, *tmp_l;
+	OutstandingRequest *request;
+
+	statlist_for_each_safe(el, &server->outstanding_requests, tmp_l) {
+		request = container_of(el, OutstandingRequest, node);
+		statlist_remove(&server->canceling_clients, el);
+		if (request->server_ps)
+			free_server_prepared_statement(request->server_ps);
+		slab_free(outstanding_request_cache, request);
+	}
+
+	free_server_prepared_statements(server);
+	varcache_clean(&server->vars);
+	slab_free(var_list_cache, server->vars.var_list);
+	slab_free(server_cache, server);
+}
+
+
 /* state change means moving between lists */
 void change_client_state(PgSocket *client, SocketState newstate)
 {
@@ -172,7 +216,7 @@ void change_client_state(PgSocket *client, SocketState newstate)
 	case CL_WAITING_LOGIN:
 		if (newstate == CL_ACTIVE)
 			newstate = CL_LOGIN;
-		/* fallthrough */
+	/* fallthrough */
 	case CL_WAITING:
 		statlist_remove(&pool->waiting_client_list, &client->head);
 		break;
@@ -194,9 +238,7 @@ void change_client_state(PgSocket *client, SocketState newstate)
 	/* put to new location */
 	switch (client->state) {
 	case CL_FREE:
-		varcache_clean(&client->vars);
-		slab_free(var_list_cache, client->vars.var_list);
-		slab_free(client_cache, client);
+		client_free(client);
 		break;
 	case CL_JUSTFREE:
 		statlist_append(&justfree_client_list, &client->head);
@@ -265,9 +307,7 @@ void change_server_state(PgSocket *server, SocketState newstate)
 	/* put to new location */
 	switch (server->state) {
 	case SV_FREE:
-		varcache_clean(&server->vars);
-		slab_free(var_list_cache, server->vars.var_list);
-		slab_free(server_cache, server);
+		server_free(server);
 		break;
 	case SV_JUSTFREE:
 		statlist_append(&justfree_server_list, &server->head);
@@ -567,6 +607,23 @@ PgDatabase *find_database(const char *name)
 	return NULL;
 }
 
+/*
+ * Similar to find_database. In case databse is not found, it will try to register
+ * it if auto-database ('*') is configured.
+ */
+PgDatabase *find_or_register_database(PgSocket *connection, const char *name)
+{
+	PgDatabase *db = find_database(name);
+	if (db == NULL) {
+		db = register_auto_database(name);
+		if (db != NULL) {
+			slog_info(connection,
+				  "registered new auto-database: %s", name);
+		}
+	}
+	return db;
+}
+
 /* find existing user */
 PgUser *find_user(const char *name)
 {
@@ -686,6 +743,24 @@ static void pause_client(PgSocket *client)
 		disconnect_client(client, true, "pause failed");
 }
 
+/*
+ * Deactivate the client socket and put it into the cancel request wait queue.
+ * We're not expecting any data from the client anymore at this point at all.
+ * But some clients might send some anyway (specifically the Go client). Since
+ * we don't care about any of that extra data we just stop reading from the
+ * socket.
+ */
+static void pause_cancel_request(PgSocket *client)
+{
+	Assert(client->state == CL_LOGIN);
+
+	slog_debug(client, "pause_cancel_request");
+	change_client_state(client, CL_WAITING_CANCEL);
+	if (!sbuf_pause(&client->sbuf))
+		disconnect_client(client, true, "pause cancel request failed");
+}
+
+
 /* wake client from wait */
 void activate_client(PgSocket *client)
 {
@@ -767,6 +842,7 @@ bool find_server(PgSocket *client)
 	if (client->link)
 		return true;
 
+	slog_noise(client, "find_server: client had no linked server yet");
 	/* try to get idle server, if allowed */
 	if (cf_pause_mode == P_PAUSE || pool->db->db_paused) {
 		server = NULL;
@@ -786,7 +862,6 @@ bool find_server(PgSocket *client)
 
 		if (!server && !check_fast_fail(client))
 			return false;
-
 	}
 	Assert(!server || server->state == SV_IDLE);
 
@@ -801,13 +876,15 @@ bool find_server(PgSocket *client)
 
 	/* link or send to waiters list */
 	if (server) {
+		slog_noise(client, "linking client to S-%p", server);
 		client->link = server;
 		server->link = client;
 		change_server_state(server, SV_ACTIVE);
 		if (varchange) {
 			server->setting_vars = true;
 			server->ready = false;
-			res = false; /* don't process client data yet */
+			res = false;	/* don't process client data yet */
+			slog_noise(client, "pausing client while applying vars");
 			if (!sbuf_pause(&client->sbuf))
 				disconnect_client(client, true, "pause failed");
 		} else {
@@ -840,6 +917,139 @@ static bool reuse_on_release(PgSocket *server)
 	return res;
 }
 
+bool queue_fake_response(PgSocket *client, char request_type)
+{
+	bool res = true;
+	PgSocket *server = client->link;
+	Assert(server);
+
+	if (request_type == 'P') {
+		slog_debug(client, "Queuing fake ParseComplete packet");
+		QUEUE_ParseComplete(res, server, client);
+	} else if (request_type == 'C') {
+		slog_debug(client, "Queuing fake CloseComplete packet");
+		QUEUE_CloseComplete(res, server, client);
+	} else {
+		fatal("Unknown fake request type %c", request_type);
+	}
+	return res;
+}
+
+/*
+ * Adds a request to the outstanding requests queue, and schedule the given
+ * action (see comments on ResponseAction for details).
+ *
+ * returns false if the required allocations failed
+ */
+bool add_outstanding_request(PgSocket *client, char type, ResponseAction action)
+{
+	OutstandingRequest *request = NULL;
+
+	PgSocket *server = client->link;
+	Assert(server);
+
+	if (action == RA_FAKE && statlist_empty(&server->outstanding_requests)) {
+		/*
+		 * If there's no outstanding requests, we can send the response
+		 * right away. And we're actually required to do that to make
+		 * sure the client receives it, because we normally only send
+		 * responses to fake requests right after we handle a response
+		 * to a real request. So if none are outstanding, we won't send
+		 * such a response.
+		 */
+		slog_noise(client, "add_outstanding_request: queueing fake response right away %c",
+			   type);
+		return queue_fake_response(client, type);
+	}
+
+	request = slab_alloc(outstanding_request_cache);
+	if (request == NULL)
+		return false;
+	request->type = type;
+	request->action = action;
+	statlist_append(&server->outstanding_requests, &request->node);
+	slog_noise(client, "add_outstanding_request: added %c, still outstanding %d",
+		   type, statlist_count(&client->link->outstanding_requests));
+	return true;
+}
+
+/*
+ * If the next outstanding request is of one of the given types, pop it off the
+ * queue. If it is of a different type, don't do anything.
+ *
+ * returns true if one of the given types was popped of off the queue.
+ */
+bool pop_outstanding_request(PgSocket *server, char *types, bool *skip)
+{
+	OutstandingRequest *request;
+	struct List *item = statlist_first(&server->outstanding_requests);
+	if (!item)
+		return false;
+
+	request = container_of(item, OutstandingRequest, node);
+	if (request->action == RA_FAKE) {
+		/*
+		 * This is weird, normally we should have already processed all fake
+		 * requests at the end of the previous packet.
+		 */
+		slog_warning(server, "pop_outstanding_request: unexpected fake request of type %c", request->type);
+		return false;
+	}
+
+	if (strchr(types, request->type) == NULL)
+		return false;
+
+	statlist_pop(&server->outstanding_requests);
+	if (skip)
+		*skip = request->action == RA_SKIP;
+	slog_noise(server, "pop_outstanding_request: popped %c, still outstanding %d, skip %d",
+		   request->type, statlist_count(&server->outstanding_requests), request->action == RA_SKIP);
+	if (request->server_ps != NULL) {
+		free_server_prepared_statement(request->server_ps);
+	}
+	slab_free(outstanding_request_cache, request);
+	return true;
+}
+
+/*
+ * clear all outstanding requests until we reach a Sync ('S') response, any
+ * Parse or Close statement requests that were still outstanding will be
+ * unregistered or re-registered from the server its cache
+ */
+bool clear_outstanding_requests_until_sync(PgSocket *server)
+{
+	struct List *item, *tmp;
+	statlist_for_each_safe(item, &server->outstanding_requests, tmp) {
+		OutstandingRequest *request = container_of(item, OutstandingRequest, node);
+		char type = request->type;
+		if (type == 'P' && request->server_ps_query_id > 0) {
+			unregister_prepared_statement(server, request->server_ps_query_id);
+			slog_noise(server,
+				   "failed prepared statement '" PREPARED_STMT_NAME_FORMAT "' removed from server cache, %d cached items",
+				   request->server_ps_query_id,
+				   HASH_COUNT(server->server_prepared_statements));
+		} else if (type == 'C' && request->server_ps != NULL) {
+			if (!add_prepared_statement(server, request->server_ps)) {
+				if (server->link)
+					disconnect_client(server->link, true, "out of memory");
+				disconnect_server(server, true, "out of memory");
+				return false;
+			}
+			slog_noise(server,
+				   "prepared statement '%s' added back to server cache, %d cached items",
+				   request->server_ps->ps->stmt_name,
+				   HASH_COUNT(server->server_prepared_statements));
+		}
+		statlist_remove(&server->outstanding_requests, item);
+		slab_free(outstanding_request_cache, request);
+
+		if (type == 'S')
+			break;
+	}
+	slog_noise(server, "clear_outstanding_requests_until_sync: still outstanding %d", statlist_count(&server->outstanding_requests));
+	return true;
+}
+
 /* send reset query */
 static bool reset_on_release(PgSocket *server)
 {
@@ -898,8 +1108,7 @@ bool release_server(PgSocket *server)
 		}
 
 		if (*cf_server_reset_query && (cf_server_reset_query_always ||
-					       pool_pool_mode(pool) == POOL_SESSION))
-		{
+					       pool_pool_mode(pool) == POOL_SESSION)) {
 			/* notify reset is required */
 			newstate = SV_TESTED;
 		} else if (cf_server_check_delay == 0 && *cf_server_check_query) {
@@ -942,6 +1151,18 @@ bool release_server(PgSocket *server)
 		return false;
 	}
 
+	if (statlist_count(&server->outstanding_requests) > 0) {
+		/*
+		 * We can't release the server if there are outstanding requests
+		 * that haven't been responded to yet, otherwise the server
+		 * might get linked to another client and it will get those
+		 * responses when it does not expect them. To be on the safe
+		 * side we simply close this connection.
+		 */
+		disconnect_server(server, true, "client disconnected with queries in progress");
+		return true;
+	}
+
 	/* enforce close request */
 	if (server->close_needed) {
 		disconnect_server(server, true, "close_needed");
@@ -991,13 +1212,14 @@ void disconnect_server(PgSocket *server, bool send_term, const char *reason, ...
 	va_end(ap);
 	reason = buf;
 
-	if (cf_log_disconnections)
+	if (cf_log_disconnections) {
 		slog_info(server, "closing because: %s (age=%" PRIu64 "s)", reason,
 			  (now - server->connect_time) / USEC);
+	}
 
 	switch (server->state) {
 	case SV_ACTIVE_CANCEL:
-	case SV_ACTIVE:	{
+	case SV_ACTIVE: {
 		PgSocket *client = server->link;
 
 		if (client) {
@@ -1027,12 +1249,10 @@ void disconnect_server(PgSocket *server, bool send_term, const char *reason, ...
 		 * usually disconnect means problems in startup phase,
 		 * except when sending cancel packet
 		 */
-		if (!server->ready)
-		{
+		if (!server->ready) {
 			server->pool->last_login_failed = true;
 			server->pool->last_connect_failed = true;
-		}
-		else
+		} else
 		{
 			/*
 			 * We did manage to connect and used the connection for query
@@ -1057,7 +1277,7 @@ void disconnect_server(PgSocket *server, bool send_term, const char *reason, ...
 
 	/* notify server and close connection */
 	if (send_term) {
-		static const uint8_t pkt_term[] = {'X', 0,0,0,4};
+		static const uint8_t pkt_term[] = {'X', 0, 0, 0, 4};
 		bool _ignore = sbuf_answer(&server->sbuf, pkt_term, sizeof(pkt_term));
 		(void) _ignore;
 	}
@@ -1102,7 +1322,6 @@ void disconnect_client(PgSocket *client, bool notify, const char *reason, ...)
 	} else {
 		disconnect_client_sqlstate(client, notify, NULL, reason);
 	}
-
 }
 
 /*
@@ -1117,9 +1336,10 @@ void disconnect_client_sqlstate(PgSocket *client, bool notify, const char *sqlst
 {
 	usec_t now = get_cached_time();
 
-	if (cf_log_disconnections && reason)
+	if (cf_log_disconnections && reason) {
 		slog_info(client, "closing because: %s (age=%" PRIu64 "s)", reason,
 			  (now - client->connect_time) / USEC);
+	}
 
 	switch (client->state) {
 	case CL_ACTIVE:
@@ -1186,7 +1406,7 @@ void disconnect_client_sqlstate(PgSocket *client, bool notify, const char *sqlst
 			 * trigger this by calling release_server again.
 			 */
 			if (canceled_server->state == SV_BEING_CANCELED
-					&& statlist_count(&canceled_server->canceling_clients) == 0 ) {
+			    && statlist_count(&canceled_server->canceling_clients) == 0) {
 				release_server(canceled_server);
 			}
 		}
@@ -1207,6 +1427,7 @@ void disconnect_client_sqlstate(PgSocket *client, bool notify, const char *sqlst
 		send_pooler_error(client, false, sqlstate, true, reason);
 	}
 
+	free_header(&client->packet_cb_state.pkt);
 	free_scram_state(&client->scram_state);
 	if (client->login_user && client->login_user->mock_auth) {
 		free(client->login_user);
@@ -1337,13 +1558,12 @@ static void dns_connect(struct PgSocket *server)
 			 */
 			sa_len = offsetof(struct sockaddr_un, sun_path) + strlen(sa_un.sun_path);
 			sa_un.sun_path[0] = '\0';
-		}
-		else {
+		} else {
 			sa_len = sizeof(sa_un);
 		}
 		sa = (struct sockaddr *)&sa_un;
 		res = 1;
-	} else if (strchr(host, ':')) {  /* assume IPv6 address on any : in addr */
+	} else if (strchr(host, ':')) {	/* assume IPv6 address on any : in addr */
 		slog_noise(server, "inet6 socket: %s", host);
 		memset(&sa_in6, 0, sizeof(sa_in6));
 		sa_in6.sin6_family = AF_INET6;
@@ -1351,7 +1571,7 @@ static void dns_connect(struct PgSocket *server)
 		sa_in6.sin6_port = htons(db->port);
 		sa = (struct sockaddr *)&sa_in6;
 		sa_len = sizeof(sa_in6);
-	} else { /* else try IPv4 */
+	} else {/* else try IPv4 */
 		slog_noise(server, "inet socket: %s", host);
 		memset(&sa_in, 0, sizeof(sa_in));
 		sa_in.sin_family = AF_INET;
@@ -1486,7 +1706,7 @@ void launch_new_connection(PgPool *pool, bool evict_if_needed)
 			goto force_new;
 
 		log_debug("launch_new_connection: peer pool full (%d >= %d)",
-				max, pool_pool_size(pool));
+			  max, pool_pool_size(pool));
 		return;
 	}
 
@@ -1518,7 +1738,7 @@ void launch_new_connection(PgPool *pool, bool evict_if_needed)
 			}
 		}
 		log_debug("launch_new_connection: pool full (%d >= %d)",
-				max, pool_pool_size(pool));
+			  max, pool_pool_size(pool));
 		return;
 	}
 
@@ -1656,7 +1876,8 @@ bool finish_client_login(PgSocket *client)
 	return true;
 }
 
-static void accept_cancel_request_for_peer(int peer_id, PgSocket *req) {
+static void accept_cancel_request_for_peer(int peer_id, PgSocket *req)
+{
 	PgDatabase *peer = NULL;
 	PgPool *pool = NULL;
 	int ttl = req->cancel_key[7] & CANCELLATION_TTL_MASK;
@@ -1698,7 +1919,7 @@ static void accept_cancel_request_for_peer(int peer_id, PgSocket *req) {
 	 * request.
 	 */
 	req->pool = pool;
-	change_client_state(req, CL_WAITING_CANCEL);
+	pause_cancel_request(req);
 
 	/*
 	 * Open a new connection over which the cancel request is forwarded to the
@@ -1726,9 +1947,10 @@ void accept_cancel_request(PgSocket *req)
 	peering_enabled = cf_peer_id > 0;
 	if (peering_enabled) {
 		/*
-		 * Extract the peer id from the cancel key
+		 * Extract the peer id from the cancel key. The peer id is
+		 * stored in the 2nd and 3rd byte.
 		 */
-		int peer_id = req->cancel_key[0] + (req->cancel_key[1] << 8);
+		int peer_id = req->cancel_key[1] + (req->cancel_key[2] << 8);
 		bool needs_forwarding_to_peer = cf_peer_id != peer_id;
 		if (needs_forwarding_to_peer) {
 			accept_cancel_request_for_peer(peer_id, req);
@@ -1806,7 +2028,7 @@ found:
 	 * request.
 	 */
 	req->pool = pool;
-	change_client_state(req, CL_WAITING_CANCEL);
+	pause_cancel_request(req);
 
 	/*
 	 * Open a new connection over which the cancel request is forwarded to the
@@ -1848,11 +2070,11 @@ bool forward_cancel_request(PgSocket *server)
 		SEND_CancelRequest(res, server, req->canceled_server->cancel_key);
 	}
 	if (!res) {
-		log_warning("sending cancel request failed: %s", strerror(errno));
+		slog_warning(req, "sending cancel request failed: %s", strerror(errno));
 		disconnect_client(req, false, "failed to send cancel request");
 		return false;
 	}
-	log_info("started sending cancel request");
+	slog_debug(req, "started sending cancel request");
 	change_client_state(req, CL_ACTIVE_CANCEL);
 	return true;
 }
@@ -2020,20 +2242,25 @@ void for_each_server(PgPool *pool, void (*func)(PgSocket *sk))
 {
 	struct List *item;
 
-	statlist_for_each(item, &pool->idle_server_list)
+	statlist_for_each(item, &pool->idle_server_list) {
 		func(container_of(item, PgSocket, head));
+	}
 
-	statlist_for_each(item, &pool->used_server_list)
+	statlist_for_each(item, &pool->used_server_list) {
 		func(container_of(item, PgSocket, head));
+	}
 
-	statlist_for_each(item, &pool->tested_server_list)
+	statlist_for_each(item, &pool->tested_server_list) {
 		func(container_of(item, PgSocket, head));
+	}
 
-	statlist_for_each(item, &pool->active_server_list)
+	statlist_for_each(item, &pool->active_server_list) {
 		func(container_of(item, PgSocket, head));
+	}
 
-	statlist_for_each(item, &pool->new_server_list)
+	statlist_for_each(item, &pool->new_server_list) {
 		func(container_of(item, PgSocket, head));
+	}
 }
 
 static void for_each_server_filtered(PgPool *pool, void (*func)(PgSocket *sk), bool (*filter)(PgSocket *sk, void *arg), void *filter_arg)
@@ -2149,7 +2376,8 @@ void tag_autodb_dirty(void)
 	}
 }
 
-static bool server_remote_addr_filter(PgSocket *sk, void *arg) {
+static bool server_remote_addr_filter(PgSocket *sk, void *arg)
+{
 	PgAddr *addr = arg;
 
 	return (pga_cmp_addr(&sk->remote_addr, addr) == 0);
@@ -2172,7 +2400,6 @@ void tag_host_addr_dirty(const char *host, const struct sockaddr *sa)
 	}
 }
 
-
 /* move objects from justfree_* to free_* lists */
 void reuse_just_freed_objects(void)
 {
@@ -2227,6 +2454,16 @@ void objects_cleanup(void)
 		kill_peer(peer);
 	}
 
+	statlist_for_each_safe(item, &justfree_server_list, tmp) {
+		PgSocket *server = container_of(item, PgSocket, head);
+		server_free(server);
+	}
+
+	statlist_for_each_safe(item, &justfree_client_list, tmp) {
+		PgSocket *client = container_of(item, PgSocket, head);
+		client_free(client);
+	}
+
 	memset(&login_client_list, 0, sizeof login_client_list);
 	memset(&user_list, 0, sizeof user_list);
 	memset(&database_list, 0, sizeof database_list);
@@ -2250,6 +2487,10 @@ void objects_cleanup(void)
 	user_cache = NULL;
 	slab_destroy(iobuf_cache);
 	iobuf_cache = NULL;
+	slab_destroy(outstanding_request_cache);
+	outstanding_request_cache = NULL;
 	slab_destroy(var_list_cache);
 	var_list_cache = NULL;
+	slab_destroy(server_prepared_statement_cache);
+	server_prepared_statement_cache = NULL;
 }
