@@ -137,6 +137,10 @@ LONG_PASSWORD = "a" * MAX_PASSWORD_LENGTH
 
 PG_SUPPORTS_SCRAM = PG_MAJOR_VERSION >= 10
 
+# psycopg.Pipeline.is_supported() does not work on rocky:8 in CI, so we create
+# our own check here that works on all our supported systems
+LIBPQ_SUPPORTS_PIPELINING = psycopg.pq.version() >= 140000
+
 
 def get_tls_support():
     with open("../config.mak", encoding="utf-8") as f:
@@ -184,6 +188,16 @@ class PortLock:
         self.lock.release()
 
 
+def notice_handler(diag: psycopg.errors.Diagnostic):
+    print(f"{diag.severity}: {diag.message_primary}")
+    if diag.message_detail:
+        print(f"DETAIL: {diag.message_detail}")
+    if diag.message_hint:
+        print(f"HINT: {diag.message_hint}")
+    if diag.context:
+        print(f"CONTEXT: {diag.context}")
+
+
 class QueryRunner:
     def __init__(self, host, port):
         self.host = host
@@ -200,18 +214,23 @@ class QueryRunner:
             options.setdefault("connect_timeout", 20)
         else:
             options.setdefault("connect_timeout", 3)
-        # needed for Ubuntu 18.04
+        # Always required for Ubuntu 18.04, but also needed for any tests
+        # involving the varcache_change database. The difference between the
+        # client_encoding specified in the config and client_encoding by the
+        # client will force a varcache change when a connectino is given.
         options.setdefault("client_encoding", "UTF8")
 
     def conn(self, *, autocommit=True, **kwargs):
         """Open a psycopg connection to this server"""
         self.set_default_connection_options(kwargs)
-        return psycopg.connect(
+        conn = psycopg.connect(
             autocommit=autocommit,
             host=self.host,
             port=self.port,
             **kwargs,
         )
+        conn.add_notice_handler(notice_handler)
+        return conn
 
     def aconn(self, *, autocommit=True, **kwargs):
         """Open an asynchronous psycopg connection to this server"""
@@ -448,6 +467,20 @@ class QueryRunner:
                 elif BSD:
                     sudo(f"pfctl -a pgbouncer_test/port_{self.port} -F all")
 
+    @contextmanager
+    def add_latency(self):
+        """Adds one second of latency to all packets to this query runner"""
+        if not LINUX:
+            raise Exception("This OS cannot run this test")
+        sudo(
+            f"tc filter add dev lo parent 1:0 protocol ip prio {self.port} u32 match ip dport {self.port} 0xffff flowid 1:2"
+        )
+        try:
+            yield
+        finally:
+            sudo(f"tc filter del dev lo parent 1: prio {self.port}")
+            pass
+
 
 class Postgres(QueryRunner):
     def __init__(self, pgdata):
@@ -602,6 +635,26 @@ class Postgres(QueryRunner):
         """
         self.sql(f"alter system set {config}")
 
+    @contextmanager
+    def log_contains(self, re_string, times=None):
+        """Checks if during this with block the log matches re_string
+
+        re_string:
+            The regex to search for.
+        times:
+            If None, any number of matches is accepted. If a number, only that
+            specific number of matches is accepted.
+        """
+        with self.log_path.open() as f:
+            f.seek(0, os.SEEK_END)
+            yield
+            content = f.read()
+            if times is None:
+                assert re.search(re_string, content)
+            else:
+                match_count = len(re.findall(re_string, content))
+                assert match_count == times
+
 
 class Bouncer(QueryRunner):
     def __init__(
@@ -663,6 +716,8 @@ class Bouncer(QueryRunner):
                 ini.write(f"logfile = {self.log_path}\n")
                 ini.write(f"auth_file = {self.auth_path}\n")
                 ini.write("pidfile = \n")
+                # Uncomment for much more noise but, more detailed debugging
+                # ini.write("verbose = 3\n")
 
                 if not USE_UNIX_SOCKETS:
                     ini.write(f"unix_socket_dir = \n")
